@@ -3,7 +3,7 @@ use crate::{misc, Basalt};
 use crossbeam::deque::{Injector, Steal};
 use crossbeam::sync::{Parker, Unparker};
 use ilmenite::{ImtImageView, ImtWeight};
-use image::{self, GenericImageView};
+use image;
 use ordered_float::OrderedFloat;
 use parking_lot::{Condvar, Mutex};
 use std::collections::HashMap;
@@ -25,7 +25,7 @@ use vulkano::image::{
 	ImageAccess, ImageCreateFlags, ImageDimensions as VkImgDimensions,
 	ImageUsage as VkImageUsage, MipmapsCount, SampleCount, StorageImage,
 };
-use vulkano::sampler::Sampler;
+use vulkano::sampler::{Sampler, SamplerCreateInfo};
 use vulkano::sync::GpuFuture;
 
 const PRINT_UPDATE_TIME: bool = false;
@@ -121,20 +121,20 @@ impl Coords {
 		}
 	}
 
-	pub fn top_left(&self) -> (f32, f32) {
-		(self.x as f32, self.y as f32)
+	pub fn top_left(&self) -> [f32; 2] {
+		[self.x as f32, self.y as f32]
 	}
 
-	pub fn top_right(&self) -> (f32, f32) {
-		((self.x + self.w) as f32, self.y as f32)
+	pub fn top_right(&self) -> [f32; 2] {
+		[(self.x + self.w) as f32, self.y as f32]
 	}
 
-	pub fn bottom_left(&self) -> (f32, f32) {
-		(self.x as f32, (self.y + self.h) as f32)
+	pub fn bottom_left(&self) -> [f32; 2] {
+		[self.x as f32, (self.y + self.h) as f32]
 	}
 
-	pub fn bottom_right(&self) -> (f32, f32) {
-		((self.x + self.w) as f32, (self.y + self.h) as f32)
+	pub fn bottom_right(&self) -> [f32; 2] {
+		[(self.x + self.w) as f32, (self.y + self.h) as f32]
 	}
 }
 
@@ -539,6 +539,7 @@ enum Command {
 }
 
 struct CommandResponse<T> {
+	tmp_response: Mutex<Option<T>>,
 	response: Mutex<Option<T>>,
 	condvar: Condvar,
 }
@@ -546,6 +547,7 @@ struct CommandResponse<T> {
 impl<T> CommandResponse<T> {
 	fn new() -> Arc<Self> {
 		Arc::new(CommandResponse {
+			tmp_response: Mutex::new(None),
 			response: Mutex::new(None),
 			condvar: Condvar::new(),
 		})
@@ -556,6 +558,17 @@ impl<T> CommandResponse<T> {
 		self.condvar.notify_one();
 	}
 
+	fn set_response(&self, val: T) {
+		*self.tmp_response.lock() = Some(val);
+	}
+
+	fn ready_response(&self) {
+		let mut tmp = self.tmp_response.lock();
+		let mut res = self.response.lock();
+		*res = tmp.take();
+		self.condvar.notify_one();
+	}
+
 	fn wait_for_response(&self) -> T {
 		let mut response = self.response.lock();
 
@@ -567,6 +580,16 @@ impl<T> CommandResponse<T> {
 	}
 }
 
+pub trait CommandResponseAbstract {
+	fn ready_response(&self);
+}
+
+impl<T> CommandResponseAbstract for CommandResponse<T> {
+	fn ready_response(&self) {
+		self.ready_response();
+	}
+}
+
 pub struct Atlas {
 	basalt: Arc<Basalt>,
 	cmd_queue: Injector<Command>,
@@ -579,33 +602,29 @@ pub struct Atlas {
 
 impl Atlas {
 	pub fn new(basalt: Arc<Basalt>) -> Arc<Self> {
-		let linear_sampler = Sampler::unnormalized(
-			basalt.device(),
-			vulkano::sampler::Filter::Linear,
-			vulkano::sampler::UnnormalizedSamplerAddressMode::ClampToBorder(
-				vulkano::sampler::BorderColor::IntTransparentBlack,
-			),
-			vulkano::sampler::UnnormalizedSamplerAddressMode::ClampToBorder(
-				vulkano::sampler::BorderColor::IntTransparentBlack,
-			),
-		)
+		let linear_sampler = Sampler::new(basalt.device(), SamplerCreateInfo {
+			mag_filter: vulkano::sampler::Filter::Linear,
+			min_filter: vulkano::sampler::Filter::Linear,
+			address_mode: [vulkano::sampler::SamplerAddressMode::ClampToBorder; 3],
+			border_color: vulkano::sampler::BorderColor::FloatTransparentBlack,
+			unnormalized_coordinates: true,
+			..SamplerCreateInfo::default()
+		})
 		.unwrap();
 
-		let nearest_sampler = Sampler::unnormalized(
-			basalt.device(),
-			vulkano::sampler::Filter::Nearest,
-			vulkano::sampler::UnnormalizedSamplerAddressMode::ClampToBorder(
-				vulkano::sampler::BorderColor::IntTransparentBlack,
-			),
-			vulkano::sampler::UnnormalizedSamplerAddressMode::ClampToBorder(
-				vulkano::sampler::BorderColor::IntTransparentBlack,
-			),
-		)
+		let nearest_sampler = Sampler::new(basalt.device(), SamplerCreateInfo {
+			mag_filter: vulkano::sampler::Filter::Nearest,
+			min_filter: vulkano::sampler::Filter::Nearest,
+			address_mode: [vulkano::sampler::SamplerAddressMode::ClampToBorder; 3],
+			border_color: vulkano::sampler::BorderColor::FloatTransparentBlack,
+			unnormalized_coordinates: true,
+			..SamplerCreateInfo::default()
+		})
 		.unwrap();
 
 		let empty_image = BstImageView::from_immutable(
 			ImmutableImage::from_iter(
-				vec![255, 255, 255, 255].into_iter(),
+				vec![1.0, 1.0, 1.0, 1.0].into_iter(),
 				VkImgDimensions::Dim2d {
 					width: 1,
 					height: 1,
@@ -646,6 +665,7 @@ impl Atlas {
 				iter_start = Instant::now();
 				let mut cmds = Vec::new();
 				let mut got_cmd = false;
+				let mut ready_on_execute: Vec<Arc<dyn CommandResponseAbstract>> = Vec::new();
 
 				loop {
 					let cmd = match atlas.cmd_queue.steal() {
@@ -697,7 +717,8 @@ impl Atlas {
 								cached_map.insert(cache_id.clone(), coords);
 							}
 
-							response.respond(Ok(coords));
+							response.set_response(Ok(coords));
+							ready_on_execute.push(response);
 
 							atlas_images[atlas_image_i - 1]
 								.insert(&region, sub_img_id, coords, up_image);
@@ -781,6 +802,10 @@ impl Atlas {
 					*atlas.image_views.lock() = Some((Instant::now(), Arc::new(draw_map)));
 				}
 
+				for response in ready_on_execute {
+					response.ready_response();
+				}
+
 				if PRINT_UPDATE_TIME && execute {
 					let mut out = format!(
 						"Atlas Updated in {:.1} ms. ",
@@ -1093,21 +1118,22 @@ impl AtlasImage {
 
 				let r_w = min_img_w - cur_img_w;
 				let r_h = cur_img_h;
-				let mut r_zeros = Vec::new();
-				r_zeros.resize((r_w * r_h * 4) as usize, 0);
-
-				let r_buf = CpuAccessibleBuffer::from_iter(
-					self.basalt.device(),
-					VkBufferUsage {
-						transfer_source: true,
-						..VkBufferUsage::none()
-					},
-					false,
-					r_zeros.into_iter(),
-				)
-				.unwrap();
 
 				if r_w > 0 && r_h > 0 {
+					let mut r_zeros = Vec::new();
+					r_zeros.resize((r_w * r_h * 4) as usize, 0);
+
+					let r_buf = CpuAccessibleBuffer::from_iter(
+						self.basalt.device(),
+						VkBufferUsage {
+							transfer_source: true,
+							..VkBufferUsage::none()
+						},
+						false,
+						r_zeros.into_iter(),
+					)
+					.unwrap();
+
 					cmd_buf
 						.copy_buffer_to_image_dimensions(
 							r_buf,
@@ -1123,21 +1149,22 @@ impl AtlasImage {
 
 				let b_w = min_img_w;
 				let b_h = min_img_h - cur_img_h;
-				let mut b_zeros = Vec::new();
-				b_zeros.resize((b_w * b_h * 4) as usize, 0);
-
-				let b_buf = CpuAccessibleBuffer::from_iter(
-					self.basalt.device(),
-					VkBufferUsage {
-						transfer_source: true,
-						..VkBufferUsage::none()
-					},
-					false,
-					b_zeros.into_iter(),
-				)
-				.unwrap();
 
 				if b_w > 0 && b_h > 0 {
+					let mut b_zeros = Vec::new();
+					b_zeros.resize((b_w * b_h * 4) as usize, 0);
+
+					let b_buf = CpuAccessibleBuffer::from_iter(
+						self.basalt.device(),
+						VkBufferUsage {
+							transfer_source: true,
+							..VkBufferUsage::none()
+						},
+						false,
+						b_zeros.into_iter(),
+					)
+					.unwrap();
+
 					cmd_buf
 						.copy_buffer_to_image_dimensions(
 							b_buf,
@@ -1245,29 +1272,31 @@ impl AtlasImage {
 			return (cmd_buf, false, cur_img_w, cur_img_h);
 		}
 
-		let upload_buf = CpuAccessibleBuffer::from_iter(
-			self.basalt.device(),
-			VkBufferUsage {
-				transfer_source: true,
-				..VkBufferUsage::none()
-			},
-			false,
-			upload_data.into_iter(),
-		)
-		.unwrap();
+		if !upload_data.is_empty() {
+			let upload_buf = CpuAccessibleBuffer::from_iter(
+				self.basalt.device(),
+				VkBufferUsage {
+					transfer_source: true,
+					..VkBufferUsage::none()
+				},
+				false,
+				upload_data.into_iter(),
+			)
+			.unwrap();
 
-		for (s, e, x, y, w, h) in copy_cmds {
-			cmd_buf
-				.copy_buffer_to_image_dimensions(
-					upload_buf.clone().into_buffer_slice().slice(s..e).unwrap(),
-					sto_img.clone(),
-					[x, y, 0],
-					[w, h, 1],
-					0,
-					1,
-					0,
-				)
-				.unwrap();
+			for (s, e, x, y, w, h) in copy_cmds {
+				cmd_buf
+					.copy_buffer_to_image_dimensions(
+						upload_buf.clone().into_buffer_slice().slice(s..e).unwrap(),
+						sto_img.clone(),
+						[x, y, 0],
+						[w, h, 1],
+						0,
+						1,
+						0,
+					)
+					.unwrap();
+			}
 		}
 
 		for (v, x, y, w, h) in copy_cmds_imt {
